﻿using C1.Win.Calendar;
using C1.Win.Calendar.DateEdit;
using C1.Win.Input;
using System.ComponentModel;
using System.Diagnostics;

namespace C1DateEditEnter
{
  public partial class ModifiedC1DateEdit : C1DateEdit
  {
    #region Fields
    
    /// <summary>
    /// Timer für Wertänderungen.
    /// </summary>
    private System.Timers.Timer timerC1DateEdit;


    /// <summary>
    /// This flag is set when "OnValueChanged" event must be ignored (e.g. if the value is set by code)
    /// </summary>
    private bool ignoreOnValueChanged = false;

    #endregion

    #region ctor
    public ModifiedC1DateEdit()
    {
      InitializeComponent();

      this.timerC1DateEdit = new System.Timers.Timer();

      ((System.ComponentModel.ISupportInitialize)(this.timerC1DateEdit)).BeginInit();
      this.SuspendLayout();
      // 
      // timerC1DateEdit
      // 
      this.timerC1DateEdit.Interval = 2000D;
      this.timerC1DateEdit.SynchronizingObject = this;
      this.timerC1DateEdit.Elapsed += new System.Timers.ElapsedEventHandler(this.timerC1DateEdit_Elapsed);

      ((System.ComponentModel.ISupportInitialize)(this.timerC1DateEdit)).EndInit();
    }
    #endregion

    #region Overridden Methods 

    protected override void OnValueChanged(EventArgs e)
    {

      Trace.WriteLine(this.Name + ".OnValueChanged: " + this.Text);


      //Maybe the event shall be ignored:
      if (this.ignoreOnValueChanged == true)
        return;

      //stop the Timer.
      this.timerC1DateEdit.Stop();

      base.OnValueChanged(e);
    }

    /// <summary>
    /// Stop the timer when control is entered.
    /// </summary>
    protected override void OnEnter(EventArgs eventargs)
    {
      this.timerC1DateEdit.Stop();

      base.OnEnter(eventargs);
    }

    /// <summary>
    /// Set/Get the value.
    /// On Set, a flag is set to ignore the "ValueChanged" event.
    /// </summary>
    public override object Value
    {
      get
      {
        return base.Value;
      }

      set
      {
        //Ignore "TextChanged"-Event:
        this.ignoreOnValueChanged = true;
        base.Value = value;

        this.ignoreOnValueChanged = false;
      }
    }


    /// <summary>
    /// Before the control is left: stop the timer.
    /// </summary>
    /// <param name="e">Argumente des Events</param>
    protected override void OnValidating(CancelEventArgs e)
    {
      //stop Timer
      this.timerC1DateEdit.Stop();

      //Our real app fires the "ValueChanged" event if the timer is still active.

      base.OnValidating(e);
    }


    // Overriding OnTextChange to parse and update the value on text changes if the ValueChangedBehavior is FireOnAnyChange.
    protected override void OnTextChanged(EventArgs e)
    {
      base.OnTextChanged(e);

      //Don't start the timer if the value is set by code:
      if (this.ignoreOnValueChanged == false)
      {
        if (ParseContent(out object parsedValue, ErrorInfo) == ErrorReason.None)
        {
          Trace.WriteLine(this.Name + ".OnTextChanged: " + this.Text);

          //DON'T set the value - this causes strange behaviour in the textbox:
          //Value = parsedValue;

          //Restart timer:
          this.timerC1DateEdit.Stop();
          this.timerC1DateEdit.Start();
        }
      }
    }


    /// <summary>
    /// Timer ticks: raise an "ValueChanged" event.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void timerC1DateEdit_Elapsed(object? sender, System.Timers.ElapsedEventArgs e)
    {
      Trace.WriteLine(this.Name + ".timerC1DateEdit_Elapsed: " + this.Text);

      this.timerC1DateEdit.Stop();

      //Restore previous editing position:
      int intSelStartBefore = this.SelectionStart;
      int intSelLengthBefore = this.SelectionLength;

      //Apply the text to the control:
      if (ParseContent(out object parsedValue, ErrorInfo) == ErrorReason.None)
      {
        this.Value = parsedValue;
      }

      this.SelectionStart = intSelStartBefore;
      this.SelectionLength = intSelLengthBefore;


      base.OnValueChanged(EventArgs.Empty);
    }

    protected override void OnKeyPress(KeyPressEventArgs e)
    {
      //Workarounds...
      if (e.KeyChar == 13)
      {
        //ENTER:
        bool canContinue = true;
        if (this.timerC1DateEdit.Enabled == true)
        {
          //Stop timer...
          this.timerC1DateEdit.Enabled = false;

          /*
          //Update the value with current text:
          if (this.UpdateValueWithCurrentText() == false)
          {
            //Enter canceled:
            //MessageBox.Show(this, "ENTER canceled - could not update text to value: " + this.c1DateEdit1.Value);
            canContinue = false;
          }
          else
          {
            //Fire "ValueChanged" before closing the form.
            this.OnValueChanged(EventArgs.Empty);
          }*/

          //Apply the text to the control:
          if (ParseContent(out object parsedValue, ErrorInfo) == ErrorReason.None)
          {
            this.Value = parsedValue;

            //Fire "ValueChanged" before closing the form.
            this.OnValueChanged(EventArgs.Empty);
          }
          else
          {
            canContinue = false;
          }
        }
        //Don't call base class code - duplicate "ValueChanged"
        //base.OnKeyPress(e);

        if (canContinue == true)
        {
          this.FindForm().AcceptButton.PerformClick();
        }
        e.Handled = true;

        //After this, another "ValueChanged" is fired.
      }
      else if (e.KeyChar == 27)
      {
        //ESCAPE:
        //TODO: first "Escape" should cancel an unfinished editing...
        //if (this.FindForm() is DialogOKAbbrechen dialogOKAbbrechen)
        //{
        //  dialogOKAbbrechen.CancelButton.PerformClick();
        //}
        base.OnKeyPress(e);
      }
      else
      {
        base.OnKeyPress(e);
      }
    }
    #endregion
  }
}